from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check UPRN
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        API_URL = "https://www.fife.gov.uk/api/custom?action=powersuite_bin_calendar_collections&actionedby=bin_calendar&loadform=true&access=citizen&locale=en"
        AUTH_URL = "https://www.fife.gov.uk/api/citizen?preview=false&locale=en"
        AUTH_KEY = "Authorization"

        r = requests.get(AUTH_URL)
        r.raise_for_status()
        auth_token = r.headers[AUTH_KEY]

        post_data = {
            "name": "bin_calendar",
            "data": {
                "uprn": user_uprn,
            },
            "email": "",
            "caseid": "",
            "xref": "",
            "xref1": "",
            "xref2": "",
        }

        headers = {
            "referer": "https://www.fife.gov.uk/services/forms/bin-calendar",
            "accept": "application/json",
            "content-type": "application/json",
            AUTH_KEY: auth_token,
        }

        r = requests.post(API_URL, data=json.dumps(post_data), headers=headers)
        r.raise_for_status()

        result = r.json()

        for collection in result["data"]["tab_collections"]:
            dict_data = {
                "type": collection["colour"],
                "collectionDate": datetime.strptime(
                    collection["date"],
                    "%A, %B %d, %Y",
                ).strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )
        return bindata



import urllib.parse

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        api_url = f"https://mycouncil.northampton.digital/BinRoundFinder?postcode={urllib.parse.quote(user_postcode)}"
        json_data = requests.get(api_url).json()

        data = {"bins": []}

        dict_data = {
            "type": json_data["type"].capitalize(),
            "collectionDate": datetime.strptime(
                json_data["date"], "%Y%m%d%H%M"
            ).strftime(date_format),
        }
        data["bins"].append(dict_data)

        return data



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://recycling.falkirk.gov.uk/api/collections/{user_uprn}"

        # Make the GET request
        response = requests.get(URI)

        # Parse the JSON response
        bin_collection = response.json()

        # Loop through each collection in bin_collection
        for collection in bin_collection["collections"]:
            bin_type = collection["type"]
            collection_dates = collection["dates"]

            # Loop through the dates for each collection type
            for date in collection_dates:
                print(f"Bin Type: {bin_type}")
                print(f"Collection Date: {date}")

                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(
                        date,
                        "%Y-%m-%d",
                    ).strftime("%d/%m/%Y"),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        SESSION_URL = "https://my.middevon.gov.uk/authapi/isauthenticated?uri=https%253A%252F%252Fmy.middevon.gov.uk%252Fen%252FAchieveForms%252F%253Fform_uri%253Dsandbox-publish%253A%252F%252FAF-Process-2289dd06-9a12-4202-ba09-857fe756f6bd%252FAF-Stage-eb382015-001c-415d-beda-84f796dbb167%252Fdefinition.json%2526redirectlink%253D%25252Fen%2526cancelRedirectLink%253D%25252Fen%2526consentMessage%253Dyes&hostname=my.middevon.gov.uk&withCredentials=true"

        API_URL = "https://my.middevon.gov.uk/apibroker/runLookup"

        payload = {
            "formValues": {
                "Your Address": {
                    "listAddress": {"value": user_uprn},
                },
            },
        }

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://my.middevon.gov.uk/fillform/?iframe_id=fillform-frame-1&db_id=",
        }

        ids = [
            "6423144f50ec0",
            "641c7ae9b4c96",
            "645e13a01dba1",
            "642315aacb919",
            "64231699483cf",
            "642421bab7478",
            "6424229605d13",
            "645e14020c9cc",
        ]

        rows_data = []

        for id in ids:
            s = requests.session()
            r = s.get(SESSION_URL)
            r.raise_for_status()
            session_data = r.json()
            sid = session_data["auth-session"]

            params = {
                "id": id,
                "repeat_against": "",
                "noRetry": "false",
                "getOnlyTokens": "undefined",
                "log_id": "",
                "app_name": "AF-Renderer::Self",
                # unix_timestamp
                "_": str(int(time.time() * 1000)),
                "sid": sid,
            }
            r = s.post(API_URL, json=payload, headers=headers, params=params)
            r.raise_for_status()
            data = r.json()
            rows_data = data["integration"]["transformed"]["rows_data"]
            if isinstance(rows_data, dict):
                date = datetime.strptime(rows_data["0"]["display"], "%d-%b-%y")
                bin_types = (rows_data["0"]["CollectionItems"]).split(" and ")

                for bin_type in bin_types:
                    dict_data = {
                        "type": bin_type,
                        "collectionDate": date.strftime(date_format),
                    }
                    bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://community.cotswold.gov.uk/s/waste-collection-enquiry"

            data = {"bins": []}

            house_number = kwargs.get("paon")
            postcode = kwargs.get("postcode")
            full_address = f"{house_number}, {postcode}"
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property
            wait = WebDriverWait(driver, 60)
            address_entry_field = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="combobox-input-19"]')
                )
            )

            address_entry_field.send_keys(str(full_address))

            address_entry_field = wait.until(
                EC.element_to_be_clickable((By.XPATH, '//*[@id="combobox-input-19"]'))
            )
            address_entry_field.click()
            address_entry_field.send_keys(Keys.BACKSPACE)
            address_entry_field.send_keys(str(full_address[len(full_address) - 1]))

            first_found_address = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, '//*[@id="dropdown-element-19"]/ul')
                )
            )

            first_found_address.click()
            # Wait for the 'Select your property' dropdown to appear and select the first result
            next_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, "//lightning-button/button"))
            )
            next_btn.click()
            bin_data = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//span[contains(text(), 'Container')]")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            rows = soup.find_all("tr", class_="slds-hint-parent")
            current_year = datetime.now().year

            for row in rows:
                columns = row.find_all("td")
                if columns:
                    container_type = row.find("th").text.strip()
                    if columns[0].get_text() == "Today":
                        collection_day = datetime.now().strftime("%a, %d %B")
                    elif columns[0].get_text() == "Tomorrow":
                        collection_day = (datetime.now() + timedelta(days=1)).strftime(
                            "%a, %d %B"
                        )
                    else:
                        collection_day = re.sub(
                            r"[^a-zA-Z0-9,\s]", "", columns[0].get_text()
                        ).strip()

                    # Parse the date from the string
                    parsed_date = datetime.strptime(collection_day, "%a, %d %B")
                    if parsed_date < datetime(
                        parsed_date.year, parsed_date.month, parsed_date.day
                    ):
                        parsed_date = parsed_date.replace(year=current_year + 1)
                    else:
                        parsed_date = parsed_date.replace(year=current_year)
                    # Format the date as %d/%m/%Y
                    formatted_date = parsed_date.strftime("%d/%m/%Y")

                    # Add the bin type and collection date to the 'data' dictionary
                    data["bins"].append(
                        {"type": container_type, "collectionDate": formatted_date}
                    )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import requests
import json
from datetime import datetime
from uk_bin_collection.uk_bin_collection.common import (
    check_uprn,
    date_format as DATE_FORMAT,
)
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete class that implements the abstract bin data fetching and parsing logic.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        url_base = (
            "https://basildonportal.azurewebsites.net/api/getPropertyRefuseInformation"
        )

        uprn = kwargs.get("uprn")
        # Check the UPRN is valid
        check_uprn(uprn)

        payload = {"uprn": uprn}

        headers = {"Content-Type": "application/json"}

        response = requests.post(url_base, data=json.dumps(payload), headers=headers)

        if response.status_code == 200:
            data = response.json()

            # Initialize an empty list to store the bin collection details
            bins = []

            # Function to add collection details to bins list
            def add_collection(service_name, collection_data):
                bins.append(
                    {
                        "type": service_name,
                        "collectionDate": collection_data.get(
                            "current_collection_date"
                        ),
                    }
                )

            available_services = data.get("refuse", {}).get("available_services", {})

            date_format = "%d-%m-%Y"  # Define the desired date format

            for service_name, service_data in available_services.items():
                # Handle the different cases of service data
                match service_data["container"]:
                    case "Green Wheelie Bin":
                        subscription_status = (
                            service_data["subscription"]["active"]
                            if service_data.get("subscription")
                            else False
                        )
                        type_descr = f"Green Wheelie Bin ({'Active' if subscription_status else 'Expired'})"
                    case "N/A":
                        type_descr = service_data.get("name", "Unknown Service")
                    case _:
                        type_descr = service_data.get("container", "Unknown Container")

                date_str = service_data.get("current_collection_date")
                if date_str:  # Ensure the date string exists
                    try:
                        # Parse and format the date string
                        date_obj = datetime.strptime(date_str, "%Y-%m-%d")
                        formatted_date = date_obj.strftime(DATE_FORMAT)
                    except ValueError:
                        formatted_date = "Invalid Date"
                else:
                    formatted_date = "No Collection Date"

                bins.append(
                    {
                        "type": type_descr,  # Use service name from the data
                        "collectionDate": formatted_date,
                    }
                )

        else:
            print(f"Failed to fetch data. Status code: {response.status_code}")
            return {}

        return {"bins": bins}



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            # Get postcode and UPRN from kwargs
            # user_postcode = kwargs.get("postcode")
            user_uprn = kwargs.get("uprn")
            # check_postcode(user_postcode)
            check_uprn(user_uprn)
            url = "https://www.stratford.gov.uk/waste-recycling/when-we-collect.cfm/part/calendar"
            payload = {
                "frmAddress1": "",
                "frmAddress2": "",
                "frmAddress3": "",
                "frmAddress4": "",
                "frmPostcode": "",
                "frmUPRN": user_uprn,
            }
            headers = {"Content-Type": "application/x-www-form-urlencoded"}

            requests.packages.urllib3.disable_warnings()
            response = requests.request("POST", url, data=payload, headers=headers)

            # Make a BS4 object
            soup = BeautifulSoup(response.content, features="html.parser")
            soup.prettify()

            # Find the table
            table = soup.find("table", class_="table")

            data = {"bins": []}

            if table:
                # Extract the column headers (bin names)
                column_headers = [
                    header.text.strip()
                    for header in table.select("thead th.text-center strong")
                ]

                # Extract the rows containing collection information
                collection_rows = table.select("tbody tr")

                # Create a dictionary to store the next date for each bin
                next_collection_dates = {bin: None for bin in column_headers}

                # Iterate through the rows
                for row in collection_rows:
                    # Get the date from the first cell
                    date_str = row.find("td").text.strip()
                    date_obj = datetime.strptime(date_str, "%A, %d/%m/%Y")

                    # Get the collection information for each bin (td elements with title attribute)
                    collection_info = [
                        cell["title"] if cell["title"] else "Not Collected"
                        for cell in row.select("td.text-center")
                    ]

                    # Iterate through each bin type and its collection date
                    for bin, status in zip(column_headers, collection_info):
                        # If the bin hasn't had a collection date yet or the new date is earlier, update it
                        if status != "Not Collected" and (
                            not next_collection_dates[bin]
                            or date_obj < next_collection_dates[bin]
                        ):
                            next_collection_dates[bin] = date_obj

                data["bins"] = [
                    {"type": bin, "collectionDate": next_date.strftime(date_format)}
                    for bin, next_date in next_collection_dates.items()
                ]
            else:
                print("Table not found in the HTML content.")
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



"""Get Bin Data

Keyword arguments:
None
"""

import json
import logging, logging.config
from abc import ABC, abstractmethod
import os
import requests
import urllib3

from uk_bin_collection.uk_bin_collection.common import update_input_json

_LOGGER = logging.getLogger(__name__)

LOGGING_CONFIG = dict(
    version=1,
    formatters={"f": {"format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s"}},
    handlers={
        "h": {"class": "logging.StreamHandler", "formatter": "f", "level": logging.INFO}
    },
    root={"handlers": ["h"], "level": logging.INFO},
)


def setup_logging(logging_config, logger_name):
    """Set up logging configuration.

    Keyword arguments:
    logging_config -- the logging configuration dictionary
    logger_name -- the name of the logger
    """
    try:
        logging.config.dictConfig(logging_config)
        logger = logging.getLogger(logger_name)
        return logger
    except Exception as exp:
        raise exp


class AbstractGetBinDataClass(ABC):
    """An abstract class that can be imported to handle the data received from the provided
    council class.

    Keyword arguments: None
    """

    def template_method(self, address_url: str, **kwargs) -> None:  # pragma: no cover
        """The main template method that is constructed

        Keyword arguments:
        address_url -- the url to get the data from
        """
        this_url = address_url
        this_local_browser = kwargs.get("local_browser", False)
        if not this_local_browser:
            kwargs["web_driver"] = kwargs.get("web_driver", None)

        bin_data_dict = self.get_and_parse_data(this_url, **kwargs)
        json_output = self.output_json(bin_data_dict)

        # if dev mode create/update council's entry in the input.json
        if kwargs.get("dev_mode"):
            self.update_dev_mode_data(
                council_module_str=kwargs.get("council_module_str"),
                this_url=this_url,
                **kwargs,
            )

        return json_output

    def get_and_parse_data(self, address_url, **kwargs):
        """Get and parse data from the URL

        Keyword arguments:
        address_url -- the URL to get the data from
        """
        if not kwargs.get("skip_get_url"):
            page = self.get_data(address_url)
            bin_data_dict = self.parse_data(page, url=address_url, **kwargs)
        else:
            bin_data_dict = self.parse_data("", url=address_url, **kwargs)

        return bin_data_dict

    def update_dev_mode_data(self, council_module_str, this_url, **kwargs):
        """Update input.json if in development mode

        Keyword arguments:
        council_module_str -- the council module string
        this_url -- the URL used
        """
        cwd = os.getcwd()
        input_file_path = os.path.join(cwd, "uk_bin_collection", "tests", "input.json")
        update_input_json(
            council_module_str,
            this_url,
            input_file_path,
            postcode=kwargs.get("postcode"),
            paon=kwargs.get("paon"),
            uprn=kwargs.get("uprn"),
            usrn=kwargs.get("usrn"),
            web_driver=kwargs.get("web_driver"),
            skip_get_url=kwargs.get("skip_get_url"),
        )

    @classmethod
    def get_data(cls, url) -> str:
        """This method makes the request to the council

        Keyword arguments:
        url -- the url to get the data from
        """
        user_agent = (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/108.0.0.0 Safari/537.36"
        )
        headers = {"User-Agent": user_agent}
        urllib3.disable_warnings(category=urllib3.exceptions.InsecureRequestWarning)

        try:
            full_page = requests.get(url, headers, verify=False, timeout=120)
            return full_page
        except requests.exceptions.RequestException as err:
            _LOGGER.error(f"Request Error: {err}")
            raise

    @abstractmethod
    def parse_data(self, page: str, **kwargs) -> dict:
        """Abstract method that takes a page as a string

        Keyword arguments:
        page -- a string from the requested page
        """

    @classmethod
    def output_json(cls, bin_data_dict: dict) -> str:
        """Method to output the json as a pretty printed string

        Keyword arguments:
        bin_data_dict -- a dict of parsed data
        """
        json_data = json.dumps(bin_data_dict, sort_keys=False, indent=4)
        return json_data



import json
import logging
import re
from datetime import datetime, timedelta

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import check_uprn, date_format
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
import urllib3


# Suppress only the single warning from urllib3 needed.
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

_LOGGER = logging.getLogger(__name__)


class CouncilClass(AbstractGetBinDataClass):
    """
    Implementation for Chesterfield Borough Council waste collection data retrieval.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        """
        Fetch and parse waste collection data for Chesterfield Borough Council.

        Args:
            page (str): Not used in this implementation.
            **kwargs: Should contain 'uprn' key.

        Returns:
            dict: Parsed bin collection data.
        """
        # Get and check UPRN
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        # Define API URLs
        API_URLS = {
            "session": "https://www.chesterfield.gov.uk/bins-and-recycling/bin-collections/check-bin-collections.aspx",
            "fwuid": "https://myaccount.chesterfield.gov.uk/anonymous/c/cbc_VE_CollectionDaysLO.app?aura.format=JSON&aura.formatAdapter=LIGHTNING_OUT",
            "search": "https://myaccount.chesterfield.gov.uk/anonymous/aura?r=2&aura.ApexAction.execute=1",
        }

        HEADERS = {
            "User-Agent": "Mozilla/5.0",
            "Content-Type": "application/x-www-form-urlencoded",
        }

        # Initialize session
        session = requests.Session()

        try:
            # Step 1: Get session
            session.get(API_URLS["session"], headers=HEADERS, verify=False)

            # Step 2: Get fwuid
            fwuid_response = session.get(
                API_URLS["fwuid"], headers=HEADERS, verify=False
            )
            fwuid_data = fwuid_response.json()
            fwuid = fwuid_data.get("auraConfig", {}).get("context", {}).get("fwuid")

            if not fwuid:
                _LOGGER.error("Failed to retrieve fwuid from the response.")
                return bindata

            # Step 3: Prepare payload for UPRN search
            payload = {
                "message": json.dumps(
                    {
                        "actions": [
                            {
                                "id": "4;a",
                                "descriptor": "aura://ApexActionController/ACTION$execute",
                                "callingDescriptor": "UNKNOWN",
                                "params": {
                                    "namespace": "",
                                    "classname": "CBC_VE_CollectionDays",
                                    "method": "getServicesByUPRN",
                                    "params": {
                                        "propertyUprn": user_uprn,
                                        "executedFrom": "Main Website",
                                    },
                                    "cacheable": False,
                                    "isContinuation": False,
                                },
                            }
                        ]
                    }
                ),
                "aura.context": json.dumps(
                    {
                        "mode": "PROD",
                        "fwuid": fwuid,
                        "app": "c:cbc_VE_CollectionDaysLO",
                        "loaded": {
                            "APPLICATION@markup://c:cbc_VE_CollectionDaysLO": "pqeNg7kPWCbx1pO8sIjdLA"
                        },
                        "dn": [],
                        "globals": {},
                        "uad": True,
                    }
                ),
                "aura.pageURI": "/bins-and-recycling/bin-collections/check-bin-collections.aspx",
                "aura.token": "null",
            }

            # Step 4: Make POST request to fetch collection data
            search_response = session.post(
                API_URLS["search"], data=payload, headers=HEADERS, verify=False
            )
            search_data = search_response.json()

            # Step 5: Extract service units
            service_units = (
                search_data.get("actions", [])[0]
                .get("returnValue", {})
                .get("returnValue", {})
                .get("serviceUnits", [])
            )

            if not service_units:
                _LOGGER.warning("No service units found for the given UPRN.")
                return bindata

            # Initialize dictionary to store bin dates
            bin_schedule = {}

            # Define icon mapping
            ICON_MAP = {
                "DOMESTIC REFUSE": "mdi:trash-can",
                "DOMESTIC RECYCLING": "mdi:recycle",
                "DOMESTIC ORGANIC": "mdi:leaf",
            }

            # Define regex pattern to capture day and date (e.g., Tue 5 Nov)
            date_pattern = re.compile(r"\b\w{3} \d{1,2} \w{3}\b")

            current_year = datetime.now().year

            # Process each service unit
            for item in service_units:
                try:
                    waste_type = item["serviceTasks"][0]["taskTypeName"]
                    waste_type = str(waste_type).replace("Collect ", "").upper()
                except (IndexError, KeyError):
                    _LOGGER.debug("Skipping a service unit due to missing data.")
                    continue

                # Extract the next scheduled date
                try:
                    dt_zulu = item["serviceTasks"][0]["serviceTaskSchedules"][0][
                        "nextInstance"
                    ]["currentScheduledDate"]
                    dt_utc = datetime.strptime(dt_zulu, "%Y-%m-%dT%H:%M:%S.%f%z")
                    dt_local = dt_utc.astimezone(None)
                    collection_date = dt_local.date()
                except (IndexError, KeyError, ValueError) as e:
                    _LOGGER.warning(f"Failed to parse date for {waste_type}: {e}")
                    continue

                # Append to bin_schedule
                bin_schedule[waste_type] = collection_date.strftime(date_format)

            # Convert bin_schedule to the expected format
            for bin_type, collection_date in bin_schedule.items():
                dict_data = {
                    "type": bin_type,
                    "collectionDate": collection_date,
                }
                bindata["bins"].append(dict_data)

            # Sort the bins by collection date
            bindata["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )

        except requests.RequestException as e:
            _LOGGER.error(f"Network error occurred: {e}")
        except json.JSONDecodeError as e:
            _LOGGER.error(f"JSON decoding failed: {e}")
        except Exception as e:
            _LOGGER.error(f"An unexpected error occurred: {e}")

        return bindata



import os
import shutil
import argparse
from common import update_input_json


def create_council(council_name: str, url: str) -> None:
    base_dir = "uk_bin_collection/uk_bin_collection"
    template_file = os.path.join(
        base_dir, "councils/council_class_template/councilclasstemplate.py"
    )
    council_dir = os.path.join(base_dir, "councils")
    test_file = os.path.join(
        "uk_bin_collection", "tests", "features", "validate_council_outputs.feature"
    )
    cwd = os.getcwd()
    input_file_path = os.path.join(cwd, "uk_bin_collection", "tests", "input.json")

    # Create council file from template
    new_council_file = os.path.join(council_dir, f"{council_name}.py")
    shutil.copy(template_file, new_council_file)

    # Update the new council class
    with open(new_council_file, "r") as file:
        filedata = file.read()

    filedata = filedata.replace("CouncilClassTemplate", council_name)

    with open(new_council_file, "w") as file:
        file.write(filedata)

    # Update the input JSON
    update_input_json(council_name, url, input_file_path)


def main():
    parser = argparse.ArgumentParser(
        description="Create a new council script from a template."
    )
    parser.add_argument(
        "council_name", type=str, help="The name of the council to be created."
    )
    parser.add_argument("url", type=str, help="The URL associated with the council.")

    args = parser.parse_args()

    create_council(args.council_name, args.url)


if __name__ == "__main__":
    main()



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        SESSION_URL = "https://my.dudley.gov.uk/authapi/isauthenticated?uri=https%253A%252F%252Fmy.dudley.gov.uk%252Fen%252FAchieveForms%252F%253Fform_uri%253Dsandbox-publish%253A%252F%252FAF-Process-373f5628-9aae-4e9e-ae09-ea7cd0588201%252FAF-Stage-52ec040b-10e6-440f-b964-23f924741496%252Fdefinition.json%2526redirectlink%253D%25252Fen%2526cancelRedirectLink%253D%25252Fen%2526consentMessage%253Dyes&hostname=my.dudley.gov.uk&withCredentials=true"

        API_URL = "https://my.dudley.gov.uk/apibroker/runLookup"

        data = {
            "formValues": {
                "My bins": {
                    "uprnToCheck": {"value": user_uprn},
                }
            },
        }

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://my.dudley.gov.uk/fillform/?iframe_id=fillform-frame-1&db_id=",
        }
        s = requests.session()
        r = s.get(SESSION_URL)
        r.raise_for_status()
        session_data = r.json()
        sid = session_data["auth-session"]
        params = {
            "id": "64899d4c2574c",
            "repeat_against": "",
            "noRetry": "true",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }
        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()
        data = r.json()
        rows_data = data["integration"]["transformed"]["rows_data"]["0"]
        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")
        BIN_TYPES = [
            ("refuseDate", "Refuse"),
            ("recyclingDate", "Recycling"),
            ("gardenDate", "Garden Waste"),
        ]
        bin_type_dict = dict(BIN_TYPES)

        for row in rows_data.items():
            if (row[0].endswith("Date")) and not row[0].endswith("EndDate"):
                if row[1]:
                    bin_type = bin_type_dict.get(row[0], row[0])
                    collection_date = datetime.strptime(row[1], "%Y-%m-%d").strftime(
                        "%d/%m/%Y"
                    )
                    dict_data = {"type": bin_type, "collectionDate": collection_date}
                    bindata["bins"].append(dict_data)

        return bindata



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        # Check the UPRN is valid
        check_uprn(uprn)

        # Request URL
        url = f"https://www.eastleigh.gov.uk/waste-bins-and-recycling/collection-dates/your-waste-bin-and-recycling-collections?uprn={uprn}"

        # Make Request
        requests.packages.urllib3.disable_warnings()
        page = requests.get(url)

        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        # Data to return
        data = {"bins": []}

        # Valid bin types
        binTypes = [
            "Household Waste Bin",
            "Recycling Bin",
            "Food Waste Bin",
            "Glass Box and Batteries",
            "Garden Waste Bin",
        ]

        # Value to create dict for DL values
        keys, values = [], []

        # Loop though DT and DD for DL containing bins
        dl = soup.find("dl", {"class": "dl-horizontal"})
        for dt in dl.find_all("dt"):
            keys.append(dt.text.strip())
        for dd in dl.find_all("dd"):
            values.append(dd.text.strip())

        # Create dict for bin name and string dates
        binDict = dict(zip(keys, values))

        # Process dict for valid bin types
        for bin in list(binDict):
            if bin in binTypes:
                if not binDict[bin].startswith("You haven't yet signed up for"):
                    # Convert date
                    date = datetime.strptime(binDict[bin], "%a, %d %b %Y")

                    # Set bin data
                    dict_data = {
                        "type": bin,
                        "collectionDate": date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

        # Return bin data
        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://digital.flintshire.gov.uk/FCC_BinDay/Home/Details2/{user_uprn}"

        # Make the GET request
        response = requests.get(URI)

        # Parse the HTML content
        soup = BeautifulSoup(response.content, "html.parser")

        # Adjust these tags and classes based on actual structure
        # Example for finding collection dates and types
        bin_collections = soup.find_all(
            "div", class_="col-md-12 col-lg-12 col-sm-12 col-xs-12"
        )  # Replace with actual class name

        # Extracting and printing the schedule data
        schedule = []
        for collection in bin_collections:
            dates = collection.find_all("div", class_="col-lg-2 col-md-2 col-sm-2")
            bin_type = collection.find("div", class_="col-lg-3 col-md-3 col-sm-3")

            if dates[0].text.strip() == "Date of Collection":
                continue

            bin_types = bin_type.text.strip().split(" / ")
            date = dates[0].text.strip()

            # Loop through the dates for each collection type
            for bin_type in bin_types:

                dict_data = {
                    "type": bin_type,
                    "collectionDate": date,
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )
        return bindata



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        # Extract data
        bin_data = {"bins": []}

        # Find the table containing the bin collection data
        table = soup.find("table", {"class": "eb-EVDNdR1G-tableContent"})

        if table:
            rows = table.find_all("tr", class_="eb-EVDNdR1G-tableRow")

            for row in rows:
                columns = row.find_all("td")
                if len(columns) >= 4:
                    collection_type = columns[1].get_text(strip=True)
                    collection_date = columns[3].get_text(strip=True)

                    # Validate collection_date format
                    if re.match(r"\d{2}/\d{2}/\d{4}", collection_date):
                        bin_entry = {
                            "type": collection_type,
                            "collectionDate": collection_date,
                        }
                        bin_data["bins"].append(bin_entry)

        return bin_data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://cag.walsall.gov.uk/BinCollections/GetBins?uprn={user_uprn}"

        headers = {
            "user-agent": "Mozilla/5.0",
        }

        response = requests.get(URI, headers=headers)

        soup = BeautifulSoup(response.text, "html.parser")
        # Extract links to collection shedule pages and iterate through the pages
        schedule_links = soup.findAll("a", {"class": "nav-link"}, href=True)
        for item in schedule_links:
            if "roundname" in item["href"]:
                # get bin colour
                bincolour = item["href"].split("=")[-1].split("%")[0].upper()
                binURL = "https://cag.walsall.gov.uk" + item["href"]
                r = requests.get(binURL, headers=headers)
                soup = BeautifulSoup(r.text, "html.parser")
                table = soup.findAll("tr")
                for tr in table:
                    td = tr.findAll("td")
                    if td:
                        dict_data = {
                            "type": bincolour,
                            "collectionDate": datetime.strptime(
                                td[1].text.strip(), "%d/%m/%Y"
                            ).strftime("%d/%m/%Y"),
                        }
                        bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://www.swindon.gov.uk/info/20122/rubbish_and_recycling_collection_days?addressList={user_uprn}&uprnSubmit=Yes"

        # Make the GET request
        response = requests.get(URI)

        # Parse the JSON response
        soup = BeautifulSoup(response.text, "html.parser")

        bin_collection_content = soup.find_all(
            "div", {"class": "bin-collection-content"}
        )
        for content in bin_collection_content:
            content_left = content.find("div", {"class": "content-left"})
            content_right = content.find("div", {"class": "content-right"})
            if content_left and content_right:

                bin_types = content_left.find("h3").text.split(" and ")
                for bin_type in bin_types:

                    collection_date = datetime.strptime(
                        content_right.find(
                            "span", {"class": "nextCollectionDate"}
                        ).text,
                        "%A, %d %B %Y",
                    ).strftime(date_format)

                    dict_data = {
                        "type": bin_type,
                        "collectionDate": collection_date,
                    }
                    bindata["bins"].append(dict_data)

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://www.westmorlandandfurness.gov.uk/bins-recycling-and-street-cleaning/waste-collection-schedule/view/{user_uprn}"

        headers = {
            "user-agent": "Mozilla/5.0",
        }

        current_year = datetime.now().year
        current_month = datetime.now().month

        response = requests.get(URI)

        soup = BeautifulSoup(response.text, "html.parser")
        # Extract links to collection shedule pages and iterate through the pages
        schedule = soup.findAll("div", {"class": "waste-collection__month"})
        for month in schedule:
            collectionmonth = datetime.strptime(month.find("h3").text, "%B")
            collectionmonth = collectionmonth.month
            collectiondays = month.findAll("li", {"class": "waste-collection__day"})
            for collectionday in collectiondays:
                day = collectionday.find(
                    "span", {"class": "waste-collection__day--day"}
                ).text.strip()
                collectiondate = datetime.strptime(day, "%d")
                collectiondate = collectiondate.replace(month=collectionmonth)
                bintype = collectionday.find(
                    "span", {"class": "waste-collection__day--colour"}
                ).text

                if (current_month > 9) and (collectiondate.month < 4):
                    collectiondate = collectiondate.replace(year=(current_year + 1))
                else:
                    collectiondate = collectiondate.replace(year=current_year)

                dict_data = {
                    "type": bintype,
                    "collectionDate": collectiondate.strftime("%d/%m/%Y"),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata
